@@ -19,6 +19,7 @@ from itertools import combinations
 from scipy.interpolate import interp1d
 from scipy.ndimage.filters import gaussian_filter
 from scipy.interpolate import UnivariateSpline as uspline
+from scipy.integrate import simpson
 
 def lsh5(hd, prefix='', top_only=False, silent=False, print_attrs=True):
     """ list the content of a HDF5 file
@@ -56,6 +57,19 @@ def create_linked_files(fn, fnlist):
         fs.close()
     ff.close()
 
+def integrate_mon(em, ts, ts0, exp):
+    """ integrate monitor counts
+        monitor counts are given by em with timestamps ts
+        ts0 is the timestamps on the exposures, with duration of exp
+    """
+    ffe = interp1d(ts, em)
+    em0 = []
+    for t in ts0:
+        tt = np.concatenate(([t], ts[(ts>t) & (ts<t+exp)], [t+exp]))
+        ee = ffe(tt)
+        em0.append(simpson(ee, tt))
+    return np.asarray(em0)/exp
+    
 def pack_d1(data, ret_trans=True):
     """ utility function to creat a list of [intensity, error] from a Data1d object 
         or from a list of Data1s objects
@@ -227,7 +241,7 @@ def proc_line_profile(queue, images, sn, nframes, detectors, qphi_range, debug,
 def proc_d1merge(args):
     """ utility function to perfrom azimuthal average and merge detectors
     """
-    images,sn,nframes,starting_frame_no,debug,detectors,qgrid,reft,save_1d,save_merged = args
+    images,sn,nframes,starting_frame_no,debug,detectors,qgrid,reft,save_1d,save_merged,dtype = args
     ret = {'merged': []}
     sc = {}
     
@@ -242,9 +256,11 @@ def proc_d1merge(args):
         for det in detectors:
             dt = Data1d()
             label = "%s_f%05d%s" % (sn, i+starting_frame_no, det.extension)
-            dt.load_from_2D(images[det.extension][i], 
-                            det.exp_para, qgrid, det.pre_process, det.exp_para.mask,
-                            save_ave=False, debug=debug, label=label)
+            dt.load_from_2D(images[det.extension][i], det.exp_para, qgrid, 
+                            pre_process=det.pre_process, flat_cor=det.flat, mask=det.exp_para.mask,
+                            save_ave=False, debug=debug, label=label, dtype=dtype)
+            if det.dark is not None:
+                dt.data -= det.dark
             dt.scale(sc[det.extension])
             ret[det.extension].append(dt)
     
@@ -422,6 +438,18 @@ class h5exp():
             ep.init_coordinates()
 
         self.save_detectors()
+
+def find_field(fh5, fieldName):
+    tstream = {}
+    samples = lsh5(fh5, top_only=True, silent=True)
+    for sn in samples:
+        for stream in list(fh5[f"{sn}"]):
+            if not 'data' in list(fh5[f"{sn}/{stream}"]):
+                continue
+            if fieldName in list(fh5[f"{sn}/{stream}/data"]):
+                tstream[sn] = stream
+                break
+    return tstream
         
 class h5xs():
     """ Scattering data in transmission geometry
@@ -440,7 +468,7 @@ class h5xs():
         self.attrs = {}
         # name of the dataset that contains transmitted beam intensity, e.g. em2_current1_mean_value
         self.transField = ''  
-        self.transStream = ''  
+        self.transStream = {}  
 
         self.fn = fn
         self.save_d1 = save_d1
@@ -453,7 +481,7 @@ class h5xs():
         self.list_samples(quiet=True)
 
         # find out what are the fields corresponding to the 2D detectors
-        # at LiX there are two possibilities
+        # at LiX there are two possibilities; assume all samples have have data stored in the same fileds
         sn = self.samples[0]
         streams = list(self.fh5[f"{sn}"])
         data_fields = {}
@@ -474,27 +502,32 @@ class h5xs():
         if self.det_name is None:
             print('fields in the h5 file: ', data_fields)
             raise Exception("Could not find the data corresponding to the detectors.")
+
+        # transStream is more complicated
+        # different samples may store the data in different streams 
         if transField=='': 
             if 'trans' in self.fh5.attrs:
                 tf = self.fh5.attrs['trans'].split(',')
-                if len(tf)<3:  # in earlier code, transStream is not recorded
-                    [v, self.transField] = tf
-                else: 
-                    [v, self.transField, self.transStream] = tf
+                # transMove, transField, transStream 
+                # but transStream is not always recorded
+                v, self.transField = tf[:2]
                 self.transMode = trans_mode(int(v))
+                self.transStream = find_field(self.fh5, self.transField)
                 return
             else:
                 self.transMode = trans_mode.from_waxs
                 self.transField = ''
-                self.transStream = ''
-        elif transField not in list(data_fields.keys()):
-            print("invalid field for transmitted intensity: ", transField)
-            raise Exception()
+                self.transStream = {}
         else:
+            try:
+                self.transStream = find_field(self.fh5, transField)
+            except:
+                print("invalid field for transmitted intensity: ", transField)
+                raise Exception()
             self.transField = transField
-            self.transStream = data_fields[transField]
             self.transMode = trans_mode.external
-        self.fh5.attrs['trans'] = ','.join([str(self.transMode.value), self.transField, self.transStream])
+            
+        self.fh5.attrs['trans'] = ','.join([str(self.transMode.value), self.transField])  #elf.transStream])
         self.fh5.flush()
             
     def save_detectors(self):
@@ -643,14 +676,14 @@ class h5xs():
         frn = self.verify_frn(sn, frn, flatten=True)
         return self.d1s[sn][group][frn]    
             
-    def get_d2(self, sn=None, det_ext=None, frn=None):
+    def get_d2(self, sn=None, det_ext=None, frn=None, dtype=None):
         if sn is None:
             sn = self.samples[0]
         d2s = {}
         for det in self.detectors:
             dset = self.fh5["%s/primary/data/%s" % (sn, self.det_name[det.extension])]
             frn = self.verify_frn(sn, frn)
-            d2 = Data2d(dset[tuple(frn)], exp=det.exp_para)
+            d2 = Data2d(dset[tuple(frn)], exp=det.exp_para, dtype=dtype)
             d2.md["frame #"] = frn 
             d2s[det.extension] = d2
         if not det_ext:
@@ -688,7 +721,7 @@ class h5xs():
         gs = fig.add_gridspec(2,2)
 
         fig.add_subplot(gs[:, 0])
-        plt.semilogy(d2.qphi_data.xc, dch1, color)
+        plt.semilogy(d2.qphi_data.xc, dch1)
         plt.semilogy(d2.qphi_data.xc, dch2)        
         
         fig.add_subplot(gs[1, 1])
@@ -744,10 +777,10 @@ class h5xs():
         
     def show_data(self, sn=None, det_ext='_SAXS', frn=None, ax=None,
                   logScale=True, showMask=False, mask_alpha=0.1, 
-                  clim=(0.1,14000), showRef=["AgBH", "r:"], cmap=None):
+                  clim=(0.1,14000), showRef=["AgBH", "r:"], cmap=None, dtype=None):
         """ display frame #frn of the data under det for sample sn
         """
-        d2 = self.get_d2(sn=sn, det_ext=det_ext, frn=frn)
+        d2 = self.get_d2(sn=sn, det_ext=det_ext, frn=frn, dtype=dtype)
         if ax is None:
             plt.figure()
             ax = plt.gca()
@@ -765,12 +798,12 @@ class h5xs():
     
     def show_data_qxy(self, sn=None, frn=None, ax=None, dq=0.006,
                       fig_type="qxy", apply_sym=False, fix_gap=False,
-                      logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None):
+                      logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None, dtype=None):
         """ display frame #frn of the data under det for sample sn
             det is a list of detectors, or a string, data file extension
             fig_type should be either "qxy" or "qphi"
         """
-        d2s = self.get_d2(sn=sn, frn=frn)
+        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype)
         if ax is None:
             plt.figure()
             ax = plt.gca()
@@ -813,8 +846,8 @@ class h5xs():
 
     def show_data_qphi(self, sn=None, frn=None, ax=None, Nq=200, Nphi=60,
                        apply_symmetry=False, fill_gap=False, interp_method='linear',
-                       logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None):
-        d2s = self.get_d2(sn=sn, frn=frn)
+                       logScale=True, useMask=True, clim=(0.1,14000), showRef=True, cmap=None, dtype=None):
+        d2s = self.get_d2(sn=sn, frn=frn, dtype=dtype)
         if ax is None:
             plt.figure()
             ax = plt.gca()
@@ -875,54 +908,98 @@ class h5xs():
         ax.set_title(f"frame #{d2s[list(d2s.keys())[0]].md['frame #']}")
 
             
-    def set_trans(self, transMode, sn=None, trigger=None, gf_sigma=5, **kwargs): 
+    def set_trans(self, transMode, sn=None, trigger=None, gf_sigma=2, dt0=-133.8, exp=1, plot_trigger=False, **kwargs): 
         """ set the transmission values for the merged data
             the trans values directly from the raw data (water peak intensity or monitor counts)
             but this value is changed if the data is scaled
             the trans value for all processed 1d data are saved as attrubutes of the dataset
+
+            when em2 is used as a monitor, a trigger must be provided to find the corresponding monitor value
+            in fly scans, this should be a motor name
+            for solution scattering, use "sol" as trigger
+
+            the timestamps on the trigger and em2 may not be synced, dt0 provides a correction to the trigger timestamp
+            if plot_trigger=True and sn is not None, will generate a plot for verification
+            
+            dt0 for sol trigger should be finite but minimal, ~0.05?
+
         """
         assert(isinstance(transMode, trans_mode))
         self.transMode = transMode
         if self.transMode==None:
             raise Exception("a valid transmited intensity mode must be specified.")
-        
+
         if sn is None:
+            if plot_trigger:
+                plot_trigger = False           # plot for a single sample only
+                print("Disabling plot_trigger since no sample name is specified.")
             samples = self.samples
         else:
             samples = [sn]
         for s in samples:
             if self.transMode==trans_mode.external: # transField should be set/validated already
                 assert(self.transField!='')
-                trans_data = self.fh5[f'{s}/{self.transStream}/data/{self.transField}'][...]
-                ts = self.fh5[f'{s}/{self.transStream}/timestamps/{self.transField}'][...]
-                if self.transStream!="primary": 
-                    # fly scanning, triggers are either from 
+                trans_data = self.fh5[f'{s}/{self.transStream[s]}/data/{self.transField}'][...]
+                ts = self.fh5[f'{s}/{self.transStream[s]}/timestamps/{self.transField}'][...]
+                if self.transStream[s]!="primary": 
+                    # fly scanning, trigger must be a motor or "sol"  
                     if trigger is None:
                         raise Exception("the motor that triggers data collection must be specified.")
-                    if trigger not in self.fh5[f'{s}/primary/timestamps'].keys():
+                    elif trigger=="sol":
+                        # in this case the monitors are read more frequently than detector exposures
+                        # integrate monitor counts based  
+                        dn = list(self.det_name.values())[0]
+                        # expect a finite but minimal offset in time since all come from the same IOC server
+                        ts0 = self.fh5[f'{s}/primary/timestamps/{dn}'][...]-dt0    
+                        dshape = self.fh5[f"{s}/primary/data/{dn}"].shape[0]   # length of the time sequence
+                        md = self.header(s)['pilatus']
+                        if 'exposure_time' in md.keys():
+                            exp=md['exposure_time']
+                        if len(ts0)==1: # multiple exposures, single trigger, as in HT measurements
+                            ts0 = ts0[0]+np.arange(dshape)*exp    
+                        trans_data1 = integrate_mon(trans_data, ts, ts0, exp)
+                    elif trigger in self.fh5[f'{s}/primary/timestamps'].keys():
+                        # intepolate instead of integrate, with filtering, less accurate
+                        trigger_ts = sef.fh5[f'{s}/primary/timestamps/{trigger}'][...]-dt0
+                        dshape = self.fh5[f"{s}/primary/data/{list(self.det_name.values())[0]}"].shape[:-2]
+                        if trigger_ts.shape != dshape:
+                            raise Exception(f"mistached timestamp length: {len(trigger_ts)} vs {dshape}")
+                        spl = uspline(ts, gaussian_filter(trans_data, gf_sigma))
+                        ts0 = trigger_ts.flatten()
+                        trans_data1 = spl(ts0)
+                    else:
                         raise Exception(f"timestamp data for {trigger} cannot be found.")
-                    trigger_ts = self.fh5[f'{s}/primary/timestamps/{trigger}'][...]
-                    dshape = self.fh5[f"{s}/primary/data/{list(self.det_name.values())[0]}"].shape[:-2]
-                    if trigger_ts.shape != dshape:
-                        raise Exception(f"mistached timestamp length: {len(trigger_ts)} vs {dshape}")
+
                     # smoothing/interpolating
-                    spl = uspline(ts, gaussian_filter(trans_data, gf_sigma))
-                    ts0 = trigger_ts.flatten()
-                    trans_data = spl(ts0)
-        
+                    if plot_trigger:
+                        plt.figure()
+                        plt.plot(ts, trans_data)
+                        plt.plot(ts0, trans_data1, "o")
+                    trans_data = trans_data1
+
             # these are the datasets that needs updated trans values
-            if 'merged' not in self.d1s[s].keys():
-                continue
+            # also need to rescale merged data, in case it's been scale, e.g. during normalization
+            grps = list(set(self.d1s[s].keys()) & set(det_model.keys()))
+            grp0 = grps[0]
+            if "merged" in self.d1s[s].keys():
+                grps += ["merged"]
+            if len(grps)==0:
+                return
+
             t_values = []
-            for i in range(len(self.d1s[s]['merged'])):
-                if self.transMode==trans_mode.external:
-                    self.d1s[s]['merged'][i].set_trans(self.transMode, trans_data[i], **kwargs)
-                else:
-                    self.d1s[s]['merged'][i].set_trans(self.transMode, **kwargs)
-                t_values.append(self.d1s[s]['merged'][i].trans)
-            if 'averaged' not in self.d1s[s].keys():
-                continue
-            self.d1s[s]['averaged'].trans = np.average(t_values)
+            for i in range(len(self.d1s[s][grp0])):
+                if "merged" in grps:
+                    idx = (self.d1s[s][grp0][i].data>0)
+                    sc0 = np.sum(self.d1s[s][grp0][i].data[idx])/np.sum(self.d1s[s]["merged"][i].data[idx])
+                    self.d1s[s]["merged"][i].scale(sc0)
+                for grp in grps:
+                    if self.transMode==trans_mode.external:
+                        self.d1s[s][grp][i].set_trans(self.transMode, trans_data[i], **kwargs)
+                    else:
+                        self.d1s[s][grp][i].set_trans(self.transMode, **kwargs)
+                t_values.append(self.d1s[s][grp0][i].trans)
+            if 'averaged' in self.d1s[s].keys():
+                self.d1s[s]['averaged'].trans = np.average(t_values)
     
     def load_d1s(self, sn=None):
         """ load the processed 1d data saved in the hdf5 file into memory 
@@ -1149,7 +1226,7 @@ class h5xs():
                                               footer=self.md_string(sn))
                 
     def load_data(self, update_only=False, detectors=None,
-           reft=-1, save_1d=False, save_merged=False, debug=False, N=8, max_c_size=0):
+           reft=-1, save_1d=False, save_merged=False, debug=False, N=8, max_c_size=0, dtype=None):
         """ assume multiple samples, parallel-process by sample
             use Pool to limit the number of processes; 
             access h5 group directly in the worker process
@@ -1211,11 +1288,11 @@ class h5xs():
 
                     if N>1: # multi-processing, need to keep track of total number of active processes                    
                         job = pool.map_async(proc_d1merge, [(images, sn, nframes, i*c_size, debug,
-                                                             detectors, self.qgrid, reft, save_1d, save_merged)])
+                                                             detectors, self.qgrid, reft, save_1d, save_merged, dtype)])
                         jobs.append(job)
                     else: # serial processing
                         [sn, fr1, data] = proc_d1merge((images, sn, nframes, i*c_size, debug, 
-                                                        detectors, self.qgrid, reft, save_1d, save_merged)) 
+                                                        detectors, self.qgrid, reft, save_1d, save_merged, dtype)) 
                         results[sn][fr1] = data                
                 else: # len(s)==4
                     for j in range(s[0]):  # slow axis
@@ -1225,11 +1302,11 @@ class h5xs():
                             images[det.extension] = dset[gn][j, i*c_size:i*c_size+nframes]
                         if N>1: # multi-processing, need to keep track of total number of active processes
                             job = pool.map_async(proc_d1merge, [(images, sn, nframes, i*c_size+j*s[1], debug,
-                                                                 detectors, self.qgrid, reft, save_1d, save_merged)])
+                                                                 detectors, self.qgrid, reft, save_1d, save_merged, dtype)])
                             jobs.append(job)
                         else: # serial processing
                             [sn, fr1, data] = proc_d1merge((images, sn, nframes, i*c_size+j*s[1], debug, 
-                                                            detectors, self.qgrid, reft, save_1d, save_merged)) 
+                                                            detectors, self.qgrid, reft, save_1d, save_merged, dtype)) 
                             results[sn][fr1] = data                
 
         if N>1:             
